import ddf.minim.*; //library for audio

//audio variable
Minim minim;
AudioInput in;

//constant
final int maxEffects = 8; //maximum number of effect
final int longueur = 800; //width of the window
final int hauteur = 533 ; //heigth of the window
final int maxImage = 15; //max number of image to display

PImage img; //will contain the image 
int imgID = 0; //id of the current image
int cx = 0; //absciss origin
int cy = 0; //ordinate origin
int r = 15; //radius
int effects = 0; //initiate effects to 0
float speed = 400.f; // initiate speed to a standard 400
boolean bool = true; //boolean used to decide clockwise or anticlockwise

int n_x ; //absciss for the modified image
int n_y; // ordinate for the modified image

/**
* This function display an image with a color filter and make it move following a circle
* @param rayon : radius of the circle
* @param r : red value
* @param g : green value
* @param b : blue value
* @param trans : transparency value
* @param vitesse : speed of the movement
**/
void movingTint( int rayon, int r, int g, int b, int trans, float vitesse){
  float t = millis()/vitesse; //compute the t used to compute the coordonate of x and y 

  tint(r, g,b, trans);//apply the filter
    n_x = (int)(cx+rayon*cos(t));
    n_y = (int)(cy+rayon*sin(t));
  image(img, n_x, n_y); //display the modified picture
}

/**
* This function apply a filter and moved the image following a circle
* @param rayon : radius of the circle
* @param trans : transparency of the new image
* @param vitesse : speed of the movement
* @param sens : TRUE clockwise / FALSE anticlockwise
* @param whichEffect : keyword to apply the right filter
**/
void movingFilter(int rayon, int trans, float vitesse,boolean sens,int whichEffect){
  float t = millis()/vitesse;


  tint(255, trans);
  if(whichEffect == POSTERIZE) //POSTERIZE needs an additionnal value
    filter(whichEffect, 4);
  else
    filter(whichEffect);
   //invert t if we want the movement to be anticlockwise
  if(sens == false){
    n_x = (int)(cx+rayon*cos(-t));
    n_y = (int)(cy+rayon*sin(-t));
  }
  else {
   n_x = (int)(cx+rayon*cos(t));
    n_y = (int)(cy+rayon*sin(t)); 
  }
 
  image(img, n_x, n_y); //display the image
}


//setup function
void setup(){
  size(longueur,hauteur); // create a window of longueur x hauteur
   minim = new Minim(this); //initate audio minim object
   minim.debugOn();
    in = minim.getLineIn(Minim.STEREO, 512); //get audio input
  img = loadImage(imgID+".jpg"); //load the first image
}

void draw(){
  background(255,255,255); //white background
  noTint();//be sure that no color will be add to the original image
  image(img,(width-img.width)/2,(height-img.height)/2); // display the image in the center of the screen
  
  switch(effects){
     case 0 : 
       movingTint(r,255,102,0,130,400.f);
     break;
     case 1 : 
       movingFilter(r,130,speed,bool,POSTERIZE);
     break;
     case 2 : 
       movingFilter(r,130,speed,bool,INVERT);
     break;
     case 3 :
       movingFilter(r,130,speed,bool,DILATE);
     break;
     case 4 :
        movingFilter(r,130,speed,bool,BLUR);
     break;
     case 5 : 
       movingFilter(r,130,speed,bool,ERODE);
     break;
     case 6 :
      movingFilter(r,130,speed,bool,THRESHOLD);
      break;
     case 7 :
       movingTint(r,255,0,255,130,400.f);
     break;
     
     default :  movingTint(r,255,102,0,130,400.f);
    
  } 
 
 //get the level of the left channel, test it and call the change function
    float boxSize;
    boxSize= in.left.level() * 100;
    
  
    if(boxSize > 14) {
      changeEffectAndImage();
    }
      
}

/**
* This function change the image and the effect randomly
**/
void changeEffectAndImage(){
  //avoid two times the same image
  int bufId = imgID;
  do{
    imgID = (int)random(maxImage);
   
  }while(imgID == bufId);
  img = loadImage(imgID+".jpg");
  
  //avoid two times the same effect
  int buf = effects;
  do{
  effects = int(random(maxEffects));
  }while(effects == buf);
  
  //compute the origin coordonate in function of the image size for the effects /w a little random
   cx = (int)random(10)+(width-img.width)/2;
   cy = (int)random(10) + (height-img.height)/2;
   r = (int)random(8) +5; //random the radius between 5 and 13
   bool = ((int)random(1) == 1) ? true : false; //random the rotation direction
   speed = random(400) + 400.f; //random the speed between 400 and 800
    
}

//this function is called when a key is pressed
void keyPressed(){
  changeEffectAndImage();
}

//When the app is closed, close the Minim audio classes
void stop()
{
  // always close Minim audio classes when you are done with them
  in.close();
  minim.stop();
  super.stop();
}
